var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { Utils } from '../../transmute-framework';
import * as InternalEventTypes from './InternalEventTypes';
export var FSATypes;
(function (FSATypes) {
    FSATypes[FSATypes["Native"] = 0] = "Native";
    FSATypes[FSATypes["SimpleKeyValue"] = 1] = "SimpleKeyValue";
    FSATypes[FSATypes["Adapter"] = 2] = "Adapter";
})(FSATypes || (FSATypes = {}));
export var getFSAType = function (fsa) {
    if (isNativeEvent(fsa)) {
        return FSATypes.Native;
    }
    if (isSimpleStringKeyValue(fsa)) {
        return FSATypes.SimpleKeyValue;
    }
    if (isAdapterEvent(fsa)) {
        return FSATypes.Adapter;
    }
    throw new Error('Unknown FSAType.');
};
export var payloadKeyToKeyType = {
    address: 'A',
    bytes32: 'B',
    uint: 'U' // not good.
};
export var isNativeEvent = function (fsa) {
    return payloadKeyToKeyType[fsa.payload.key] !== undefined && fsa.payload.value !== undefined;
};
export var isSimpleStringKeyValue = function (fsa) {
    var keys = Object.keys(fsa.payload);
    return (keys.length === 2 &&
        fsa.payload.key !== undefined &&
        fsa.payload.value !== undefined &&
        fsa.payload.key.length < 32 &&
        fsa.payload.value.length < 32);
};
export var isAdapterEvent = function (fsa) {
    return payloadKeyToKeyType[fsa.payload.key] === undefined;
};
export var getFSAsFromReceipt = function (receipt) {
    var fsa = [];
    receipt.logs.forEach(function (event) {
        if (event.event === 'EsEvent') {
            fsa.push(esEventToFSA(event.args));
        }
    });
    return fsa;
};
export var filterEventsByMeta = function (events, prop, propValue) {
    var fsa = [];
    events.forEach(function (event) {
        if (event.meta[prop] === propValue) {
            fsa.push(event);
        }
    });
    return fsa;
};
export var valuesToEsEvent = function (Id, TxOrigin, MsgSender, Created, EventType, KeyType, ValueType, Key, Value) {
    return {
        Id: Id,
        TxOrigin: TxOrigin,
        MsgSender: MsgSender,
        Created: Created,
        EventType: EventType,
        KeyType: KeyType,
        ValueType: ValueType,
        Key: Key,
        Value: Value
    };
};
export var getFSAFromEsEventWithPartial = function (esEvent, partialFSA) {
    var payload;
    switch (partialFSA.meta.valueType) {
        case 'A':
            payload = {
                key: 'address',
                value: Utils.toChecksumAddress('0x' + esEvent.Value.split('0x000000000000000000000000')[1])
            };
            break;
        case 'B':
            payload = {
                key: 'bytes32',
                value: esEvent.Value
            };
            break;
        case 'U':
            payload = {
                key: 'uint',
                value: Utils.hexToNumber(esEvent.Value)
            };
            break;
        case 'S':
            payload = {
                key: Utils.toAscii(esEvent.Key),
                value: Utils.toAscii(esEvent.Value)
            };
            break;
    }
    return __assign({}, partialFSA, { payload: payload });
};
export var esEventToFSA = function (esEvent) {
    var partialFSA = {
        type: Utils.toAscii(esEvent.EventType),
        payload: {},
        meta: {
            keyType: Utils.toAscii(esEvent.KeyType),
            valueType: Utils.toAscii(esEvent.ValueType),
            id: esEvent.Id.toNumber(),
            created: esEvent.Created.toNumber(),
            txOrigin: Utils.toChecksumAddress(esEvent.TxOrigin),
            msgSender: Utils.toChecksumAddress(esEvent.MsgSender),
            isInternal: InternalEventTypes.ALL.indexOf(Utils.toAscii(esEvent.EventType)) !== -1
        }
    };
    return getFSAFromEsEventWithPartial(esEvent, partialFSA);
};
export var arrayToFSA = function (values) {
    var esEvent = valuesToEsEvent(values[0], values[1], values[2], values[3], values[4], values[5], values[6], values[7], values[8]);
    return esEventToFSA(esEvent);
};
//# sourceMappingURL=EventTransformer.js.map