"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var transmute_framework_1 = require("../../transmute-framework");
exports.initialState = {
    readModelStoreKey: '',
    readModelType: 'PackageManager',
    contractAddress: '0x0000000000000000000000000000000000000000',
    lastEvent: null,
    model: {} // where all the updates from events will be made,
};
var updatesFromMeta = function (meta) {
    return {
        lastEvent: meta.id
    };
};
var calculateCostFromRefs = function (refs) {
    var keys = Object.keys(refs);
    var size = 0;
    keys.forEach(function (key) {
        size += refs[key];
    });
    return size;
};
var handlePackageUpdate = function (state, action) {
    var meta = action.meta;
    state.model.adapterMeta = state.model.adapterMeta || {};
    state.model.adapterMeta[meta.valueType] = state.model.adapterMeta[meta.valueType] || {
        eventRefs: {},
        cumulativeEventSize: 0,
        packageRefs: {},
        cumulativePackageSize: 0
    };
    var thisEventAdapterMeta = state.model.adapterMeta[meta.valueType];
    var eventRefs = thisEventAdapterMeta.eventRefs;
    var packageRefs = thisEventAdapterMeta.packageRefs;
    var eventRef = meta.adapterMeta.eventHash;
    var eventSize = meta.adapterMeta.eventSize;
    var packageRef = meta.adapterMeta.packageHash;
    var packageSize = meta.adapterMeta.packageSize;
    eventRefs[eventRef] = eventSize;
    packageRefs[packageRef] = packageSize;
    state.model[packageRef] = action.payload.name;
    thisEventAdapterMeta.cumulativeEventSize = calculateCostFromRefs(eventRefs);
    thisEventAdapterMeta.cumulativePackageSize = calculateCostFromRefs(packageRefs);
    return state.model;
};
var handlePackageDelete = function (state, action) {
    var meta = action.meta;
    state.model.adapterMeta = state.model.adapterMeta;
    var thisEventAdapterMeta = state.model.adapterMeta[meta.valueType];
    var eventRefs = state.model.adapterMeta[meta.valueType].eventRefs;
    var packageRefs = state.model.adapterMeta[meta.valueType].packageRefs;
    var eventRef = meta.adapterMeta.eventHash;
    var eventSize = meta.adapterMeta.eventSize;
    eventRefs[eventRef] = eventSize;
    var packageRef = meta.adapterMeta.packageHash;
    delete packageRefs[packageRef];
    delete state.model[packageRef];
    thisEventAdapterMeta.cumulativeEventSize = calculateCostFromRefs(eventRefs);
    thisEventAdapterMeta.cumulativePackageSize = calculateCostFromRefs(packageRefs);
    return state.model;
};
var handlers = (_a = {},
    _a[transmute_framework_1.InternalEventTypes.NEW_OWNER] = function (state, action) {
        // console.log(action);
        return __assign({}, state, { model: __assign({}, state.model, { owner: action.payload.value }) }, updatesFromMeta(action.meta));
    },
    _a[transmute_framework_1.InternalEventTypes.WL_SET] = function (state, action) {
        // console.log(action);
        return __assign({}, state, { model: __assign({}, state.model, { whitelisted: true }) }, updatesFromMeta(action.meta));
    },
    _a.PACKAGE_UPDATED = function (state, action) {
        // console.log('delets seem bust ', action)
        // make sure to settle object before returning it.
        var newState = __assign({}, state, { model: handlePackageUpdate(state, action) }, updatesFromMeta(action.meta));
        return newState;
    },
    _a.PACKAGE_DELETED = function (state, action) {
        // make sure to settle object before returning it.
        // console.log("package deleted events must be interceptor so that meta can be extracted...");
        // console.log(action.meta);
        var newState = __assign({}, state, { model: handlePackageDelete(state, action) }, updatesFromMeta(action.meta));
        return newState;
    },
    _a);
exports.reducer = function (state, action) {
    if (handlers[action.type]) {
        state = handlers[action.type](state, action);
    }
    return state;
};
var _a;
//# sourceMappingURL=reducer.js.map