"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var transmute_framework_1 = require("../../transmute-framework");
var InternalEventTypes = require("./InternalEventTypes");
var FSATypes;
(function (FSATypes) {
    FSATypes[FSATypes["Native"] = 0] = "Native";
    FSATypes[FSATypes["SimpleKeyValue"] = 1] = "SimpleKeyValue";
    FSATypes[FSATypes["Adapter"] = 2] = "Adapter";
})(FSATypes = exports.FSATypes || (exports.FSATypes = {}));
exports.getFSAType = function (fsa) {
    if (exports.isNativeEvent(fsa)) {
        return FSATypes.Native;
    }
    if (exports.isSimpleStringKeyValue(fsa)) {
        return FSATypes.SimpleKeyValue;
    }
    if (exports.isAdapterEvent(fsa)) {
        return FSATypes.Adapter;
    }
    throw new Error('Unknown FSAType.');
};
exports.payloadKeyToKeyType = {
    address: 'A',
    bytes32: 'B',
    uint: 'U' // not good.
};
exports.isNativeEvent = function (fsa) {
    return exports.payloadKeyToKeyType[fsa.payload.key] !== undefined && fsa.payload.value !== undefined;
};
exports.isSimpleStringKeyValue = function (fsa) {
    var keys = Object.keys(fsa.payload);
    return (keys.length === 2 &&
        fsa.payload.key !== undefined &&
        fsa.payload.value !== undefined &&
        fsa.payload.key.length < 32 &&
        fsa.payload.value.length < 32);
};
exports.isAdapterEvent = function (fsa) {
    return exports.payloadKeyToKeyType[fsa.payload.key] === undefined;
};
exports.getFSAsFromReceipt = function (receipt) {
    var fsa = [];
    receipt.logs.forEach(function (event) {
        if (event.event === 'EsEvent') {
            fsa.push(exports.esEventToFSA(event.args));
        }
    });
    return fsa;
};
exports.filterEventsByMeta = function (events, prop, propValue) {
    var fsa = [];
    events.forEach(function (event) {
        if (event.meta[prop] === propValue) {
            fsa.push(event);
        }
    });
    return fsa;
};
exports.valuesToEsEvent = function (Id, TxOrigin, MsgSender, Created, EventType, KeyType, ValueType, Key, Value) {
    return {
        Id: Id,
        TxOrigin: TxOrigin,
        MsgSender: MsgSender,
        Created: Created,
        EventType: EventType,
        KeyType: KeyType,
        ValueType: ValueType,
        Key: Key,
        Value: Value
    };
};
exports.getFSAFromEsEventWithPartial = function (esEvent, partialFSA) {
    var payload;
    switch (partialFSA.meta.valueType) {
        case 'A':
            payload = {
                key: 'address',
                value: transmute_framework_1.Utils.toChecksumAddress('0x' + esEvent.Value.split('0x000000000000000000000000')[1])
            };
            break;
        case 'B':
            payload = {
                key: 'bytes32',
                value: esEvent.Value
            };
            break;
        case 'U':
            payload = {
                key: 'uint',
                value: transmute_framework_1.Utils.hexToNumber(esEvent.Value)
            };
            break;
        case 'S':
            payload = {
                key: transmute_framework_1.Utils.toAscii(esEvent.Key),
                value: transmute_framework_1.Utils.toAscii(esEvent.Value)
            };
            break;
    }
    return __assign({}, partialFSA, { payload: payload });
};
exports.esEventToFSA = function (esEvent) {
    var partialFSA = {
        type: transmute_framework_1.Utils.toAscii(esEvent.EventType),
        payload: {},
        meta: {
            keyType: transmute_framework_1.Utils.toAscii(esEvent.KeyType),
            valueType: transmute_framework_1.Utils.toAscii(esEvent.ValueType),
            id: esEvent.Id.toNumber(),
            created: esEvent.Created.toNumber(),
            txOrigin: transmute_framework_1.Utils.toChecksumAddress(esEvent.TxOrigin),
            msgSender: transmute_framework_1.Utils.toChecksumAddress(esEvent.MsgSender),
            isInternal: InternalEventTypes.ALL.indexOf(transmute_framework_1.Utils.toAscii(esEvent.EventType)) !== -1
        }
    };
    return exports.getFSAFromEsEventWithPartial(esEvent, partialFSA);
};
exports.arrayToFSA = function (values) {
    var esEvent = exports.valuesToEsEvent(values[0], values[1], values[2], values[3], values[4], values[5], values[6], values[7], values[8]);
    return exports.esEventToFSA(esEvent);
};
//# sourceMappingURL=EventTransformer.js.map