"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var moment_1 = require("moment");
var transmute_framework_1 = require("../../../../transmute-framework");
var states_1 = require("../__mocks__/states");
var jsonLogic_1 = require("../__mocks__/jsonLogic");
/**
 * Logic tests
 */
describe('Logic event tests', function () {
    it('Logic.applyJsonLogic returns the result of jsonLogic.apply', function () {
        // Here we inject a date and isHeroBirthday property into each event
        // We then use json logic to evaluate a rule on each event
        // This approach is useful for systems where user's design complex rules about state
        var eventsOverTime = lodash_1.cloneDeep(states_1.default);
        var birthday = states_1.default[0].payload.hero.birthday;
        var date = moment_1.default(birthday, 'YYYY-MM-DD').subtract(2, 'days');
        eventsOverTime.forEach(function (event) {
            event.payload.date = date.format('YYYY-MM-DD');
            date = date.add(1, 'days');
            var check = transmute_framework_1.Logic.applyJsonLogic(jsonLogic_1.isHeroBirthday, event.payload);
            event.payload.isHeroBirthday = transmute_framework_1.Logic.applyJsonLogic(jsonLogic_1.isHeroBirthday, event.payload);
            if (check) {
                expect(eventsOverTime.indexOf(event)).toBe(2);
            }
        });
    });
    it('Logic.applyJsonLogicProjection map events to applyJsonLogic(rule, event)', function () {
        var eventsOverTime = lodash_1.cloneDeep(states_1.default);
        var birthday = states_1.default[0].payload.hero.birthday;
        var date = moment_1.default(birthday, 'YYYY-MM-DD').subtract(2, 'days');
        eventsOverTime.forEach(function (event) {
            event.payload.date = date.format('YYYY-MM-DD');
            date = date.add(1, 'days');
        });
        var states2 = eventsOverTime.map(function (event) {
            return event.payload;
        });
        var projection = transmute_framework_1.Logic.applyJsonLogicProjection(jsonLogic_1.isHeroBirthday, states2);
        expect(!projection[1]);
        expect(projection[2]);
    });
});
//# sourceMappingURL=logic.tests.js.map